use eframe::egui::{self, Response, RichText, Ui, Widget};

use super::{model_response::ModelResponse, state::AppState};

pub struct ModelSelection<'a> {
    app_state: &'a mut AppState,
}

impl<'a> ModelSelection<'a> {
    pub fn new(app_state: &'a mut AppState) -> Self {
        Self { app_state }
    }
}

impl<'a> Widget for ModelSelection<'a> {
    fn ui(self, ui: &mut Ui) -> Response {
        ui.heading("Models");
        ui.horizontal(|ui| {
            egui::ComboBox::from_label("")
                .selected_text("Model List")
                .show_ui(ui, |ui| {
                    for model in &self.app_state.models_available {
                        let selected_models = self
                            .app_state
                            .selected_models
                            .iter()
                            .map(|m| m.name.clone())
                            .collect::<Vec<String>>();
                        let contained = selected_models.contains(&model);

                        let label = ui.selectable_label(contained, model);

                        if label.clicked() && !contained {
                            self.app_state.selected_models.push(ModelResponse::new(
                                model.clone(),
                                "No message".to_string(),
                                0,
                                0.0,
                            ));
                        }
                    }
                });

            let selection_frame = egui::Frame::none()
                .inner_margin(4.0)
                .rounding(4.0)
                .fill(egui::Color32::DARK_GRAY);

            if self.app_state.selected_models.len() == 0 {
                selection_frame.show(ui, |ui| {
                    ui.label(RichText::new("No models selected").strong());
                });

                return;
            }

            let mut model_to_remove: Option<String> = None;

            for model in &self.app_state.selected_models {
                selection_frame.show(ui, |ui| {
                    ui.horizontal(|ui| {
                        ui.label(RichText::new(model.name.clone()).strong());
                        if ui.button("X").clicked() {
                            model_to_remove.replace(model.name.clone());
                        }
                    });
                });
            }

            if let Some(model) = model_to_remove {
                self.app_state.selected_models.retain(|m| m.name != model);
            }
        })
        .response
    }
}
